(** Time-stamp: <modified the 09/06/2008 (at 10:05) by Erwan Jahier> *)


(** This module defines functions that translate SyntaxTreeCore datatypes into
    CompiledData one. 

    Basically, it 

    - calls [EvalConst.f] wherever it is necessary, i.e., for items
    that are required to be static, such as arrays sizes, or array
    indexes.  

    - recursively call itself for translating sub-terms
*)

val typ  : CompiledData.id_solver -> SyntaxTreeCore.type_exp  -> CompiledData.type_eff
val clock: CompiledData.id_solver -> SyntaxTreeCore.clock_exp -> CompiledData.clock_eff

(**
   A [node_exp] is a name plus a list of static arguments.

   The goal of [node] is to 
   - compute the effective type of static arguments
   - check they are compatible with the node signature 
   check the type of the static arguments (
*)
val node : CompiledData.id_solver -> SyntaxTreeCore.node_exp Lxm.srcflagged -> 
  CompiledData.node_exp_eff

val eq : CompiledData.id_solver -> SyntaxTreeCore.eq_info Lxm.srcflagged -> 
  CompiledData.eq_info_eff Lxm.srcflagged

val assertion : CompiledData.id_solver -> SyntaxTreeCore.val_exp  Lxm.srcflagged -> 
  CompiledData.val_exp_eff  Lxm.srcflagged


(** Useful to type check node aliased by array iterators *)
val translate_predef_static_args: 
  CompiledData.id_solver -> SyntaxTreeCore.static_arg Lxm.srcflagged list -> 
  Lxm.t -> CompiledData.static_arg_eff list
